@@ -42,8 +42,6 @@ import numpy as np
 
 from scipy import sparse
 
-from joblib import Memory
-
 from sklearn.decomposition import randomized_svd
 from urllib.request import urlopen
 
@@ -74,8 +72,6 @@ for url, filename in resources:
 # #############################################################################
 # Loading the redirect files
 
-memory = Memory(cachedir=".")
-
 
 def index(redirects, index_map, k):
     """Find the index of an article name after redirect resolution"""
@@ -124,8 +120,6 @@ def get_redirects(redirects_filename):
     return redirects
 
 
-# disabling joblib as the pickling of large dicts seems much too slow
-#@memory.cache
 def get_adjacency_matrix(redirects_filename, page_links_filename, limit=None):
     """Extract the adjacency graph as a scipy sparse matrix
 
