@@ -78,7 +78,7 @@ def plot_calibration_curve(est, name, fig_index):
     sigmoid = CalibratedClassifierCV(est, cv=2, method='sigmoid')
 
     # Logistic regression with no calibration as baseline
-    lr = LogisticRegression(C=1., solver='lbfgs')
+    lr = LogisticRegression(C=1.)
 
     fig = plt.figure(fig_index, figsize=(10, 10))
     ax1 = plt.subplot2grid((3, 1), (0, 0), rowspan=2)
