@@ -12,7 +12,7 @@ print(__doc__)
 
 from sklearn.cluster import AffinityPropagation
 from sklearn import metrics
-from sklearn.datasets.samples_generator import make_blobs
+from sklearn.datasets import make_blobs
 
 # #############################################################################
 # Generate sample data
@@ -35,8 +35,7 @@ print("V-measure: %0.3f" % metrics.v_measure_score(labels_true, labels))
 print("Adjusted Rand Index: %0.3f"
       % metrics.adjusted_rand_score(labels_true, labels))
 print("Adjusted Mutual Information: %0.3f"
-      % metrics.adjusted_mutual_info_score(labels_true, labels,
-                                           average_method='arithmetic'))
+      % metrics.adjusted_mutual_info_score(labels_true, labels))
 print("Silhouette Coefficient: %0.3f"
       % metrics.silhouette_score(X, labels, metric='sqeuclidean'))
 
