@@ -21,7 +21,6 @@ example is based on the Boston housing data set.
 import numpy as np
 import matplotlib
 import matplotlib.pyplot as plt
-from distutils.version import LooseVersion
 
 print(__doc__)
 
@@ -34,10 +33,11 @@ from sklearn.model_selection import train_test_split
 from sklearn.linear_model import RidgeCV
 from sklearn.compose import TransformedTargetRegressor
 from sklearn.metrics import median_absolute_error, r2_score
+from sklearn.utils.fixes import parse_version
 
 
 # `normed` is being deprecated in favor of `density` in histograms
-if LooseVersion(matplotlib.__version__) >= '2.1':
+if parse_version(matplotlib.__version__) >= parse_version('2.1'):
     density_param = {'density': True}
 else:
     density_param = {'normed': True}
@@ -138,7 +138,9 @@ target = np.array(dataset.feature_names) == "DIS"
 X = dataset.data[:, np.logical_not(target)]
 y = dataset.data[:, target].squeeze()
 y_trans = quantile_transform(dataset.data[:, target],
-                             output_distribution='normal').squeeze()
+                             n_quantiles=300,
+                             output_distribution='normal',
+                             copy=True).squeeze()
 
 ###############################################################################
 # A :class:`sklearn.preprocessing.QuantileTransformer` is used such that the
@@ -184,7 +186,8 @@ ax0.set_ylim([0, 10])
 
 regr_trans = TransformedTargetRegressor(
     regressor=RidgeCV(),
-    transformer=QuantileTransformer(output_distribution='normal'))
+    transformer=QuantileTransformer(n_quantiles=300,
+                                    output_distribution='normal'))
 regr_trans.fit(X_train, y_train)
 y_pred = regr_trans.predict(X_test)
 
