@@ -41,15 +41,20 @@ print('explained variance ratio (first two components): %s'
       % str(pca.explained_variance_ratio_))
 
 plt.figure()
-for c, i, target_name in zip("rgb", [0, 1, 2], target_names):
-    plt.scatter(X_r[y == i, 0], X_r[y == i, 1], c=c, label=target_name)
-plt.legend()
+colors = ['navy', 'turquoise', 'darkorange']
+lw = 2
+
+for color, i, target_name in zip(colors, [0, 1, 2], target_names):
+    plt.scatter(X_r[y == i, 0], X_r[y == i, 1], color=color, alpha=.8, lw=lw,
+                label=target_name)
+plt.legend(loc='best', shadow=False, scatterpoints=1)
 plt.title('PCA of IRIS dataset')
 
 plt.figure()
-for c, i, target_name in zip("rgb", [0, 1, 2], target_names):
-    plt.scatter(X_r2[y == i, 0], X_r2[y == i, 1], c=c, label=target_name)
-plt.legend()
+for color, i, target_name in zip(colors, [0, 1, 2], target_names):
+    plt.scatter(X_r2[y == i, 0], X_r2[y == i, 1], alpha=.8, color=color,
+                label=target_name)
+plt.legend(loc='best', shadow=False, scatterpoints=1)
 plt.title('LDA of IRIS dataset')
 
 plt.show()
