@@ -39,16 +39,17 @@ X = np.sort(200 * rng.rand(600, 1) - 100, axis=0)
 y = np.array([np.pi * np.sin(X).ravel(), np.pi * np.cos(X).ravel()]).T
 y += (0.5 - rng.rand(*y.shape))
 
-X_train, X_test, y_train, y_test = train_test_split(X, y,
-                                                    train_size=400,
-                                                    random_state=4)
+X_train, X_test, y_train, y_test = train_test_split(
+    X, y, train_size=400, test_size=200, random_state=4)
 
 max_depth = 30
-regr_multirf = MultiOutputRegressor(RandomForestRegressor(max_depth=max_depth,
+regr_multirf = MultiOutputRegressor(RandomForestRegressor(n_estimators=100,
+                                                          max_depth=max_depth,
                                                           random_state=0))
 regr_multirf.fit(X_train, y_train)
 
-regr_rf = RandomForestRegressor(max_depth=max_depth, random_state=2)
+regr_rf = RandomForestRegressor(n_estimators=100, max_depth=max_depth,
+                                random_state=2)
 regr_rf.fit(X_train, y_train)
 
 # Predict on new data
