@@ -10,7 +10,7 @@ print(__doc__)
 
 import matplotlib.pyplot as plt
 from sklearn.svm import SVC
-from sklearn.cross_validation import StratifiedKFold
+from sklearn.model_selection import StratifiedKFold
 from sklearn.feature_selection import RFECV
 from sklearn.datasets import make_classification
 
@@ -23,7 +23,7 @@ X, y = make_classification(n_samples=1000, n_features=25, n_informative=3,
 svc = SVC(kernel="linear")
 # The "accuracy" scoring is proportional to the number of correct
 # classifications
-rfecv = RFECV(estimator=svc, step=1, cv=StratifiedKFold(y, 2),
+rfecv = RFECV(estimator=svc, step=1, cv=StratifiedKFold(2),
               scoring='accuracy')
 rfecv.fit(X, y)
 
