@@ -30,7 +30,7 @@ import numpy as np
 
 from matplotlib import pyplot as plt
 
-from sklearn.metrics.classification import accuracy_score, log_loss
+from sklearn.metrics import accuracy_score, log_loss
 from sklearn.gaussian_process import GaussianProcessClassifier
 from sklearn.gaussian_process.kernels import RBF
 
