@@ -20,9 +20,9 @@ import matplotlib.pyplot as plt
 from sklearn.linear_model import lasso_path, enet_path
 from sklearn import datasets
 
-diabetes = datasets.load_diabetes()
-X = diabetes.data
-y = diabetes.target
+
+X, y = datasets.load_diabetes(return_X_y=True)
+
 
 X /= X.std(axis=0)  # Standardize data (easier to set the l1_ratio parameter)
 
@@ -31,11 +31,11 @@ X /= X.std(axis=0)  # Standardize data (easier to set the l1_ratio parameter)
 eps = 5e-3  # the smaller it is the longer is the path
 
 print("Computing regularization path using the lasso...")
-alphas_lasso, coefs_lasso, _ = lasso_path(X, y, eps, fit_intercept=False)
+alphas_lasso, coefs_lasso, _ = lasso_path(X, y, eps=eps, fit_intercept=False)
 
 print("Computing regularization path using the positive lasso...")
 alphas_positive_lasso, coefs_positive_lasso, _ = lasso_path(
-    X, y, eps, positive=True, fit_intercept=False)
+    X, y, eps=eps, positive=True, fit_intercept=False)
 print("Computing regularization path using the elastic net...")
 alphas_enet, coefs_enet, _ = enet_path(
     X, y, eps=eps, l1_ratio=0.8, fit_intercept=False)
@@ -47,8 +47,6 @@ alphas_positive_enet, coefs_positive_enet, _ = enet_path(
 # Display results
 
 plt.figure(1)
-ax = plt.gca()
-
 colors = cycle(['b', 'r', 'g', 'c', 'k'])
 neg_log_alphas_lasso = -np.log10(alphas_lasso)
 neg_log_alphas_enet = -np.log10(alphas_enet)
@@ -64,7 +62,6 @@ plt.axis('tight')
 
 
 plt.figure(2)
-ax = plt.gca()
 neg_log_alphas_positive_lasso = -np.log10(alphas_positive_lasso)
 for coef_l, coef_pl, c in zip(coefs_lasso, coefs_positive_lasso, colors):
     l1 = plt.plot(neg_log_alphas_lasso, coef_l, c=c)
@@ -78,7 +75,6 @@ plt.axis('tight')
 
 
 plt.figure(3)
-ax = plt.gca()
 neg_log_alphas_positive_enet = -np.log10(alphas_positive_enet)
 for (coef_e, coef_pe, c) in zip(coefs_enet, coefs_positive_enet, colors):
     l1 = plt.plot(neg_log_alphas_enet, coef_e, c=c)
