@@ -36,7 +36,7 @@ X += .3 * np.random.normal(size=n_samples)
 X = X[:, np.newaxis]
 
 # Fit the classifier
-clf = linear_model.LogisticRegression(C=1e5, solver='lbfgs')
+clf = linear_model.LogisticRegression(C=1e5)
 clf.fit(X, y)
 
 # and plot the result
