@@ -39,20 +39,19 @@ from sklearn import linear_model
 X = 1. / (np.arange(1, 11) + np.arange(0, 10)[:, np.newaxis])
 y = np.ones(10)
 
-###############################################################################
+# #############################################################################
 # Compute paths
 
 n_alphas = 200
 alphas = np.logspace(-10, -2, n_alphas)
-clf = linear_model.Ridge(fit_intercept=False)
 
 coefs = []
 for a in alphas:
-    clf.set_params(alpha=a)
-    clf.fit(X, y)
-    coefs.append(clf.coef_)
+    ridge = linear_model.Ridge(alpha=a, fit_intercept=False)
+    ridge.fit(X, y)
+    coefs.append(ridge.coef_)
 
-###############################################################################
+# #############################################################################
 # Display results
 
 ax = plt.gca()
