@@ -12,14 +12,14 @@ print(__doc__)
 import numpy as np
 import matplotlib.pyplot as plt
 from sklearn.linear_model import SGDClassifier
-from sklearn.datasets.samples_generator import make_blobs
+from sklearn.datasets import make_blobs
 
 # we create 50 separable points
 X, Y = make_blobs(n_samples=50, centers=2, random_state=0, cluster_std=0.60)
 
 # fit the model
-clf = SGDClassifier(loss="hinge", alpha=0.01, max_iter=200,
-                    fit_intercept=True, tol=1e-3)
+clf = SGDClassifier(loss="hinge", alpha=0.01, max_iter=200)
+
 clf.fit(X, Y)
 
 # plot the line, the points, and the nearest vectors to the plane
