@@ -22,7 +22,7 @@ Axes3D
 # Locally linear embedding of the swiss roll
 
 from sklearn import manifold, datasets
-X, color = datasets.samples_generator.make_swiss_roll(n_samples=1500)
+X, color = datasets.make_swiss_roll(n_samples=1500)
 
 print("Computing LLE embedding")
 X_r, err = manifold.locally_linear_embedding(X, n_neighbors=12,
