@@ -46,9 +46,11 @@ a = plt.contour(xx, yy, Z, levels=[0], linewidths=2, colors='darkred')
 plt.contourf(xx, yy, Z, levels=[0, Z.max()], colors='palevioletred')
 
 s = 40
-b1 = plt.scatter(X_train[:, 0], X_train[:, 1], c='white', s=s)
-b2 = plt.scatter(X_test[:, 0], X_test[:, 1], c='blueviolet', s=s)
-c = plt.scatter(X_outliers[:, 0], X_outliers[:, 1], c='gold', s=s)
+b1 = plt.scatter(X_train[:, 0], X_train[:, 1], c='white', s=s, edgecolors='k')
+b2 = plt.scatter(X_test[:, 0], X_test[:, 1], c='blueviolet', s=s,
+                 edgecolors='k')
+c = plt.scatter(X_outliers[:, 0], X_outliers[:, 1], c='gold', s=s,
+                edgecolors='k')
 plt.axis('tight')
 plt.xlim((-5, 5))
 plt.ylim((-5, 5))
