@@ -8,22 +8,16 @@ from scipy.special import psi
 import pytest
 
 from sklearn.decomposition import LatentDirichletAllocation
-from sklearn.decomposition._online_lda import (_dirichlet_expectation_1d,
-                                               _dirichlet_expectation_2d)
+from sklearn.decomposition._lda import (_dirichlet_expectation_1d,
+                                        _dirichlet_expectation_2d)
 
-from sklearn.utils.testing import assert_allclose
-from sklearn.utils.testing import assert_true
-from sklearn.utils.testing import assert_equal
-from sklearn.utils.testing import assert_array_almost_equal
-from sklearn.utils.testing import assert_almost_equal
-from sklearn.utils.testing import assert_greater_equal
-from sklearn.utils.testing import assert_raises_regexp
-from sklearn.utils.testing import if_safe_multiprocessing_with_blas
-from sklearn.utils.testing import assert_warns
+from sklearn.utils._testing import assert_allclose
+from sklearn.utils._testing import assert_array_almost_equal
+from sklearn.utils._testing import assert_almost_equal
+from sklearn.utils._testing import if_safe_multiprocessing_with_blas
 
 from sklearn.exceptions import NotFittedError
-from sklearn.externals.six.moves import xrange
-from sklearn.externals.six import StringIO
+from io import StringIO
 
 
 def _build_sparse_mtx():
@@ -65,7 +59,7 @@ def test_lda_fit_batch():
     for component in lda.components_:
         # Find top 3 words in each LDA component
         top_idx = set(component.argsort()[-3:][::-1])
-        assert_true(tuple(sorted(top_idx)) in correct_idx_grps)
+        assert tuple(sorted(top_idx)) in correct_idx_grps
 
 
 def test_lda_fit_online():
@@ -81,7 +75,7 @@ def test_lda_fit_online():
     for component in lda.components_:
         # Find top 3 words in each LDA component
         top_idx = set(component.argsort()[-3:][::-1])
-        assert_true(tuple(sorted(top_idx)) in correct_idx_grps)
+        assert tuple(sorted(top_idx)) in correct_idx_grps
 
 
 def test_lda_partial_fit():
@@ -92,13 +86,13 @@ def test_lda_partial_fit():
     lda = LatentDirichletAllocation(n_components=n_components,
                                     learning_offset=10., total_samples=100,
                                     random_state=rng)
-    for i in xrange(3):
+    for i in range(3):
         lda.partial_fit(X)
 
     correct_idx_grps = [(0, 1, 2), (3, 4, 5), (6, 7, 8)]
     for c in lda.components_:
         top_idx = set(c.argsort()[-3:][::-1])
-        assert_true(tuple(sorted(top_idx)) in correct_idx_grps)
+        assert tuple(sorted(top_idx)) in correct_idx_grps
 
 
 def test_lda_dense_input():
@@ -113,7 +107,7 @@ def test_lda_dense_input():
     for component in lda.components_:
         # Find top 3 words in each LDA component
         top_idx = set(component.argsort()[-3:][::-1])
-        assert_true(tuple(sorted(top_idx)) in correct_idx_grps)
+        assert tuple(sorted(top_idx)) in correct_idx_grps
 
 
 def test_lda_transform():
@@ -125,7 +119,7 @@ def test_lda_transform():
     lda = LatentDirichletAllocation(n_components=n_components,
                                     random_state=rng)
     X_trans = lda.fit_transform(X)
-    assert_true((X_trans > 0.0).any())
+    assert (X_trans > 0.0).any()
     assert_array_almost_equal(np.sum(X_trans, axis=1),
                               np.ones(X_trans.shape[0]))
 
@@ -154,8 +148,8 @@ def test_lda_partial_fit_dim_mismatch():
                                     learning_offset=5., total_samples=20,
                                     random_state=rng)
     lda.partial_fit(X_1)
-    assert_raises_regexp(ValueError, r"^The provided data has",
-                         lda.partial_fit, X_2)
+    with pytest.raises(ValueError, match=r"^The provided data has"):
+        lda.partial_fit(X_2)
 
 
 def test_invalid_params():
@@ -171,7 +165,8 @@ def test_invalid_params():
     )
     for param, model in invalid_models:
         regex = r"^Invalid %r parameter" % param
-        assert_raises_regexp(ValueError, regex, model.fit, X)
+        with pytest.raises(ValueError, match=regex):
+            model.fit(X)
 
 
 def test_lda_negative_input():
@@ -179,17 +174,20 @@ def test_lda_negative_input():
     X = np.full((5, 10), -1.)
     lda = LatentDirichletAllocation()
     regex = r"^Negative values in data passed"
-    assert_raises_regexp(ValueError, regex, lda.fit, X)
+    with pytest.raises(ValueError, match=regex):
+        lda.fit(X)
 
 
 def test_lda_no_component_error():
-    # test `transform` and `perplexity` before `fit`
+    # test `perplexity` before `fit`
     rng = np.random.RandomState(0)
     X = rng.randint(4, size=(20, 10))
     lda = LatentDirichletAllocation()
-    regex = r"^no 'components_' attribute"
-    assert_raises_regexp(NotFittedError, regex, lda.transform, X)
-    assert_raises_regexp(NotFittedError, regex, lda.perplexity, X)
+    regex = ("This LatentDirichletAllocation instance is not fitted yet. "
+             "Call 'fit' with appropriate arguments before using this "
+             "estimator.")
+    with pytest.raises(NotFittedError, match=regex):
+        lda.perplexity(X)
 
 
 def test_lda_transform_mismatch():
@@ -202,8 +200,8 @@ def test_lda_transform_mismatch():
     lda = LatentDirichletAllocation(n_components=n_components,
                                     random_state=rng)
     lda.partial_fit(X)
-    assert_raises_regexp(ValueError, r"^The provided data has",
-                         lda.partial_fit, X_2)
+    with pytest.raises(ValueError, match=r"^The provided data has"):
+        lda.partial_fit(X_2)
 
 
 @if_safe_multiprocessing_with_blas
@@ -220,7 +218,7 @@ def test_lda_multi_jobs(method):
     correct_idx_grps = [(0, 1, 2), (3, 4, 5), (6, 7, 8)]
     for c in lda.components_:
         top_idx = set(c.argsort()[-3:][::-1])
-        assert_true(tuple(sorted(top_idx)) in correct_idx_grps)
+        assert tuple(sorted(top_idx)) in correct_idx_grps
 
 
 @if_safe_multiprocessing_with_blas
@@ -237,7 +235,7 @@ def test_lda_partial_fit_multi_jobs():
     correct_idx_grps = [(0, 1, 2), (3, 4, 5), (6, 7, 8)]
     for c in lda.components_:
         top_idx = set(c.argsort()[-3:][::-1])
-        assert_true(tuple(sorted(top_idx)) in correct_idx_grps)
+        assert tuple(sorted(top_idx)) in correct_idx_grps
 
 
 def test_lda_preplexity_mismatch():
@@ -252,13 +250,12 @@ def test_lda_preplexity_mismatch():
     lda.fit(X)
     # invalid samples
     invalid_n_samples = rng.randint(4, size=(n_samples + 1, n_components))
-    assert_raises_regexp(ValueError, r'Number of samples',
-                         lda._perplexity_precomp_distr, X, invalid_n_samples)
+    with pytest.raises(ValueError, match=r'Number of samples'):
+        lda._perplexity_precomp_distr(X, invalid_n_samples)
     # invalid topic number
     invalid_n_components = rng.randint(4, size=(n_samples, n_components + 1))
-    assert_raises_regexp(ValueError, r'Number of topics',
-                         lda._perplexity_precomp_distr, X,
-                         invalid_n_components)
+    with pytest.raises(ValueError, match=r'Number of topics'):
+        lda._perplexity_precomp_distr(X, invalid_n_components)
 
 
 @pytest.mark.parametrize('method', ('online', 'batch'))
@@ -277,11 +274,11 @@ def test_lda_perplexity(method):
 
     lda_2.fit(X)
     perp_2 = lda_2.perplexity(X, sub_sampling=False)
-    assert_greater_equal(perp_1, perp_2)
+    assert perp_1 >= perp_2
 
     perp_1_subsampling = lda_1.perplexity(X, sub_sampling=True)
     perp_2_subsampling = lda_2.perplexity(X, sub_sampling=True)
-    assert_greater_equal(perp_1_subsampling, perp_2_subsampling)
+    assert perp_1_subsampling >= perp_2_subsampling
 
 
 @pytest.mark.parametrize('method', ('online', 'batch'))
@@ -300,7 +297,7 @@ def test_lda_score(method):
 
     lda_2.fit_transform(X)
     score_2 = lda_2.score(X)
-    assert_greater_equal(score_2, score_1)
+    assert score_2 >= score_1
 
 
 def test_perplexity_input_format():
@@ -347,19 +344,6 @@ def test_lda_fit_perplexity():
     assert_almost_equal(perplexity1, perplexity2)
 
 
-def test_doc_topic_distr_deprecation():
-    # Test that the appropriate warning message is displayed when a user
-    # attempts to pass the doc_topic_distr argument to the perplexity method
-    n_components, X = _build_sparse_mtx()
-    lda = LatentDirichletAllocation(n_components=n_components, max_iter=1,
-                                    learning_method='batch',
-                                    total_samples=100, random_state=0)
-    distr1 = lda.fit_transform(X)
-    distr2 = None
-    assert_warns(DeprecationWarning, lda.perplexity, X, distr1)
-    assert_warns(DeprecationWarning, lda.perplexity, X, distr2)
-
-
 def test_lda_empty_docs():
     """Test LDA on empty document (all-zero rows)."""
     Z = np.zeros((5, 4))
@@ -400,8 +384,8 @@ def check_verbosity(verbose, evaluate_every, expected_lines,
 
     n_lines = out.getvalue().count('\n')
     n_perplexity = out.getvalue().count('perplexity')
-    assert_equal(expected_lines, n_lines)
-    assert_equal(expected_perplexities, n_perplexity)
+    assert expected_lines == n_lines
+    assert expected_perplexities == n_perplexity
 
 
 @pytest.mark.parametrize(
@@ -415,9 +399,3 @@ def test_verbosity(verbose, evaluate_every, expected_lines,
                    expected_perplexities):
     check_verbosity(verbose, evaluate_every, expected_lines,
                     expected_perplexities)
-
-
-def test_lda_n_topics_deprecation():
-    n_components, X = _build_sparse_mtx()
-    lda = LatentDirichletAllocation(n_topics=10, learning_method='batch')
-    assert_warns(DeprecationWarning, lda.fit, X)
