@@ -16,6 +16,7 @@ from scipy import sparse
 from numpy.lib.stride_tricks import as_strided
 
 from ..utils import check_array, check_random_state, deprecated
+from ..utils.validation import _deprecate_positional_args
 from ..base import BaseEstimator
 
 __all__ = ['PatchExtractor',
@@ -33,11 +34,11 @@ def _make_edges_3d(n_x, n_y, n_z=1):
 
     Parameters
     ----------
-    n_x : integer
+    n_x : int
         The size of the grid in the x direction.
-    n_y : integer
+    n_y : int
         The size of the grid in the y direction.
-    n_z : integer, optional
+    n_z : integer, default=1
         The size of the grid in the z direction, defaults to 1
     """
     vertices = np.arange(n_x * n_y * n_z).reshape((n_x, n_y, n_z))
@@ -129,7 +130,8 @@ def _to_graph(n_x, n_y, n_z, mask=None, img=None,
     return return_as(graph)
 
 
-def img_to_graph(img, mask=None, return_as=sparse.coo_matrix, dtype=None):
+@_deprecate_positional_args
+def img_to_graph(img, *, mask=None, return_as=sparse.coo_matrix, dtype=None):
     """Graph of the pixel-to-pixel gradient connections
 
     Edges are weighted with the gradient values.
@@ -138,14 +140,16 @@ def img_to_graph(img, mask=None, return_as=sparse.coo_matrix, dtype=None):
 
     Parameters
     ----------
-    img : ndarray, 2D or 3D
-        2D or 3D image
-    mask : ndarray of booleans, optional
+    img : ndarray of shape (height, width) or (height, width, channel)
+        2D or 3D image.
+    mask : ndarray of shape (height, width) or \
+            (height, width, channel), dtype=bool, default=None
         An optional mask of the image, to consider only part of the
         pixels.
-    return_as : np.ndarray or a sparse matrix class, optional
+    return_as : np.ndarray or a sparse matrix class, \
+            default=sparse.coo_matrix
         The class to use to build the returned adjacency matrix.
-    dtype : None or dtype, optional
+    dtype : dtype, default=None
         The data of the returned sparse matrix. By default it is the
         dtype of img
 
@@ -163,7 +167,8 @@ def img_to_graph(img, mask=None, return_as=sparse.coo_matrix, dtype=None):
     return _to_graph(n_x, n_y, n_z, mask, img, return_as, dtype)
 
 
-def grid_to_graph(n_x, n_y, n_z=1, mask=None, return_as=sparse.coo_matrix,
+@_deprecate_positional_args
+def grid_to_graph(n_x, n_y, n_z=1, *, mask=None, return_as=sparse.coo_matrix,
                   dtype=np.int):
     """Graph of the pixel-to-pixel connections
 
@@ -175,14 +180,15 @@ def grid_to_graph(n_x, n_y, n_z=1, mask=None, return_as=sparse.coo_matrix,
         Dimension in x axis
     n_y : int
         Dimension in y axis
-    n_z : int, optional, default 1
+    n_z : int, default=1
         Dimension in z axis
-    mask : ndarray of booleans, optional
+    mask : ndarray of shape (n_x, n_y, n_z), dtype=bool, default=None
         An optional mask of the image, to consider only part of the
         pixels.
-    return_as : np.ndarray or a sparse matrix class, optional
+    return_as : np.ndarray or a sparse matrix class, \
+            default=sparse.coo_matrix
         The class to use to build the returned adjacency matrix.
-    dtype : dtype, optional, default int
+    dtype : dtype, default=int
         The data of the returned sparse matrix. By default it is int
 
     Notes
@@ -216,7 +222,7 @@ def _compute_n_patches(i_h, i_w, p_h, p_w, max_patches=None):
         The height of a patch
     p_w : int
         The width of a patch
-    max_patches : integer or float, optional default is None
+    max_patches : int or float, default=None
         The maximum number of patches to extract. If max_patches is a float
         between 0 and 1, it is taken to be a proportion of the total number
         of patches.
@@ -257,12 +263,12 @@ def _extract_patches(arr, patch_shape=8, extraction_step=1):
     arr : ndarray
         n-dimensional array of which patches are to be extracted
 
-    patch_shape : integer or tuple of length arr.ndim
+    patch_shape : int or tuple of length arr.ndim.default=8
         Indicates the shape of the patches to be extracted. If an
         integer is given, the shape will be a hypercube of
         sidelength given by its value.
 
-    extraction_step : integer or tuple of length arr.ndim
+    extraction_step : int or tuple of length arr.ndim, default=1
         Indicates step size at which extraction shall be performed.
         If integer is given, then the step is uniform in all dimensions.
 
@@ -298,6 +304,7 @@ def _extract_patches(arr, patch_shape=8, extraction_step=1):
     patches = as_strided(arr, shape=shape, strides=strides)
     return patches
 
+
 @deprecated("The function feature_extraction.image.extract_patches has been "
             "deprecated in 0.22 and will be removed in 0.24.")
 def extract_patches(arr, patch_shape=8, extraction_step=1):
@@ -316,12 +323,12 @@ def extract_patches(arr, patch_shape=8, extraction_step=1):
     arr : ndarray
         n-dimensional array of which patches are to be extracted
 
-    patch_shape : integer or tuple of length arr.ndim
+    patch_shape : int or tuple of length arr.ndim, default=8
         Indicates the shape of the patches to be extracted. If an
         integer is given, the shape will be a hypercube of
         sidelength given by its value.
 
-    extraction_step : integer or tuple of length arr.ndim
+    extraction_step : int or tuple of length arr.ndim, default=1
         Indicates step size at which extraction shall be performed.
         If integer is given, then the step is uniform in all dimensions.
 
@@ -339,7 +346,9 @@ def extract_patches(arr, patch_shape=8, extraction_step=1):
                             extraction_step=extraction_step)
 
 
-def extract_patches_2d(image, patch_size, max_patches=None, random_state=None):
+@_deprecate_positional_args
+def extract_patches_2d(image, patch_size, *, max_patches=None,
+                       random_state=None):
     """Reshape a 2D image into a collection of patches
 
     The resulting patches are allocated in a dedicated array.
@@ -348,20 +357,20 @@ def extract_patches_2d(image, patch_size, max_patches=None, random_state=None):
 
     Parameters
     ----------
-    image : array, shape = (image_height, image_width) or
+    image : ndarray of shape (image_height, image_width) or \
         (image_height, image_width, n_channels)
         The original image data. For color images, the last dimension specifies
         the channel: a RGB image would have `n_channels=3`.
 
-    patch_size : tuple of ints (patch_height, patch_width)
-        the dimensions of one patch
+    patch_size : tuple of int (patch_height, patch_width)
+        The dimensions of one patch.
 
-    max_patches : integer or float, optional default is None
-        The maximum number of patches to extract. If max_patches is a float
+    max_patches : int or float, default=None
+        The maximum number of patches to extract. If `max_patches` is a float
         between 0 and 1, it is taken to be a proportion of the total number
         of patches.
 
-    random_state : int, RandomState instance or None, optional (default=None)
+    random_state : int, RandomState instance, default=None
         Determines the random number generator used for random sampling when
         `max_patches` is not None. Use an int to make the randomness
         deterministic.
@@ -369,7 +378,7 @@ def extract_patches_2d(image, patch_size, max_patches=None, random_state=None):
 
     Returns
     -------
-    patches : array, shape = (n_patches, patch_height, patch_width) or
+    patches : array of shape (n_patches, patch_height, patch_width) or \
         (n_patches, patch_height, patch_width, n_channels)
         The collection of patches extracted from the image, where `n_patches`
         is either `max_patches` or the total number of patches that can be
@@ -445,20 +454,20 @@ def reconstruct_from_patches_2d(patches, image_size):
 
     Parameters
     ----------
-    patches : array, shape = (n_patches, patch_height, patch_width) or
+    patches : ndarray of shape (n_patches, patch_height, patch_width) or \
         (n_patches, patch_height, patch_width, n_channels)
         The complete set of patches. If the patches contain colour information,
         channels are indexed along the last dimension: RGB patches would
         have `n_channels=3`.
 
-    image_size : tuple of ints (image_height, image_width) or
+    image_size : tuple of int (image_height, image_width) or \
         (image_height, image_width, n_channels)
-        the size of the image that will be reconstructed
+        The size of the image that will be reconstructed.
 
     Returns
     -------
-    image : array, shape = image_size
-        the reconstructed image
+    image : ndarray of shape image_size
+        The reconstructed image.
     """
     i_h, i_w = image_size[:2]
     p_h, p_w = patches.shape[1:3]
@@ -483,23 +492,24 @@ class PatchExtractor(BaseEstimator):
 
     Read more in the :ref:`User Guide <image_feature_extraction>`.
 
+    .. versionadded:: 0.9
+
     Parameters
     ----------
-    patch_size : tuple of ints (patch_height, patch_width)
-        the dimensions of one patch
+    patch_size : tuple of int (patch_height, patch_width)
+        The dimensions of one patch.
 
-    max_patches : integer or float, optional default is None
+    max_patches : int or float, default=None
         The maximum number of patches per image to extract. If max_patches is a
         float in (0, 1), it is taken to mean a proportion of the total number
         of patches.
 
-    random_state : int, RandomState instance or None, optional (default=None)
+    random_state : int, RandomState instance, default=None
         Determines the random number generator used for random sampling when
         `max_patches` is not None. Use an int to make the randomness
         deterministic.
         See :term:`Glossary <random_state>`.
 
-
     Examples
     --------
     >>> from sklearn.datasets import load_sample_images
@@ -514,21 +524,22 @@ class PatchExtractor(BaseEstimator):
     >>> print('Patches shape: {}'.format(pe_trans.shape))
     Patches shape: (545706, 2, 2)
     """
-
-    def __init__(self, patch_size=None, max_patches=None, random_state=None):
+    @_deprecate_positional_args
+    def __init__(self, *, patch_size=None, max_patches=None,
+                 random_state=None):
         self.patch_size = patch_size
         self.max_patches = max_patches
         self.random_state = random_state
 
     def fit(self, X, y=None):
-        """Do nothing and return the estimator unchanged
+        """Do nothing and return the estimator unchanged.
 
         This method is just there to implement the usual API and hence
         work in pipelines.
 
         Parameters
         ----------
-        X : array-like, shape [n_samples, n_features]
+        X : array-like of shape (n_samples, n_features)
             Training data.
         """
         return self
@@ -538,7 +549,7 @@ class PatchExtractor(BaseEstimator):
 
         Parameters
         ----------
-        X : array, shape = (n_samples, image_height, image_width) or
+        X : ndarray of shape (n_samples, image_height, image_width) or \
             (n_samples, image_height, image_width, n_channels)
             Array of images from which to extract patches. For color images,
             the last dimension specifies the channel: a RGB image would have
@@ -546,7 +557,7 @@ class PatchExtractor(BaseEstimator):
 
         Returns
         -------
-        patches : array, shape = (n_patches, patch_height, patch_width) or
+        patches : array of shape (n_patches, patch_height, patch_width) or \
              (n_patches, patch_height, patch_width, n_channels)
              The collection of patches extracted from the images, where
              `n_patches` is either `n_samples * max_patches` or the total
@@ -572,7 +583,8 @@ class PatchExtractor(BaseEstimator):
         patches = np.empty(patches_shape)
         for ii, image in enumerate(X):
             patches[ii * n_patches:(ii + 1) * n_patches] = extract_patches_2d(
-                image, patch_size, self.max_patches, self.random_state)
+                image, patch_size, max_patches=self.max_patches,
+                random_state=self.random_state)
         return patches
 
     def _more_tags(self):
