@@ -10,7 +10,7 @@ extern "C" {
 struct svm_node
 {
 	int dim;
-	int ind; /* index. A bit redundanat, but needed if using a
+	int ind; /* index. A bit redundant, but needed if using a
                     precomputed kernel */
 	double *values;
 };
@@ -79,7 +79,7 @@ struct svm_model
 	int *sv_ind;            /* index of support vectors */
 
 	double *rho;		/* constants in decision functions (rho[k*(k-1)/2]) */
-	double *probA;		/* pariwise probability information */
+	double *probA;		/* pairwise probability information */
 	double *probB;
 
 	/* for classification only */
@@ -104,7 +104,7 @@ struct svm_csr_model
         int *sv_ind;            /* index of support vectors */
 
 	double *rho;		/* constants in decision functions (rho[k*(k-1)/2]) */
-	double *probA;		/* pariwise probability information */
+	double *probA;		/* pairwise probability information */
 	double *probB;
 
 	/* for classification only */
