@@ -1,4 +1,3 @@
-from __future__ import division
 
 import pickle
 from io import BytesIO
@@ -11,18 +10,19 @@ from sklearn.datasets import load_digits, load_iris
 from sklearn.model_selection import train_test_split
 from sklearn.model_selection import cross_val_score
 
-from sklearn.externals.six.moves import zip
-from sklearn.utils.testing import assert_almost_equal
-from sklearn.utils.testing import assert_array_equal
-from sklearn.utils.testing import assert_array_almost_equal
-from sklearn.utils.testing import assert_equal
-from sklearn.utils.testing import assert_raises
-from sklearn.utils.testing import assert_raise_message
-from sklearn.utils.testing import assert_greater
-from sklearn.utils.testing import assert_warns
+from sklearn.utils._testing import assert_almost_equal
+from sklearn.utils._testing import assert_array_equal
+from sklearn.utils._testing import assert_array_almost_equal
+from sklearn.utils._testing import assert_raises
+from sklearn.utils._testing import assert_raise_message
+from sklearn.utils._testing import assert_warns
+from sklearn.utils._testing import assert_no_warnings
 
 from sklearn.naive_bayes import GaussianNB, BernoulliNB
 from sklearn.naive_bayes import MultinomialNB, ComplementNB
+from sklearn.naive_bayes import CategoricalNB
+from sklearn.naive_bayes import BaseNB, BaseDiscreteNB
+
 
 # Data is just 6 separable points in the plane
 X = np.array([[-2, -1], [-1, -1], [-1, -2], [1, 1], [1, 2], [2, 1]])
@@ -118,11 +118,11 @@ def test_gnb_priors():
 def test_gnb_priors_sum_isclose():
     # test whether the class prior sum is properly tested"""
     X = np.array([[-1, -1], [-2, -1], [-3, -2], [-4, -5], [-5, -4],
-                 [1, 1], [2, 1], [3, 2], [4, 4], [5, 5]])
+                  [1, 1], [2, 1], [3, 2], [4, 4], [5, 5]])
     priors = np.array([0.08, 0.14, 0.03, 0.16, 0.11, 0.16, 0.07, 0.14,
                        0.11, 0.0])
     Y = np.array([1, 2, 3, 4, 5, 6, 7, 8, 9, 10])
-    clf = GaussianNB(priors)
+    clf = GaussianNB(priors=priors)
     # smoke test for issue #9633
     clf.fit(X, Y)
 
@@ -144,10 +144,10 @@ def test_gnb_prior_large_bias():
     """Test if good prediction when class prior favor largely one class"""
     clf = GaussianNB(priors=np.array([0.01, 0.99]))
     clf.fit(X, y)
-    assert_equal(clf.predict([[-0.1, -0.1]]), np.array([2]))
+    assert clf.predict([[-0.1, -0.1]]) == np.array([2])
 
 
-def test_check_update_with_no_data():
+def test_gnb_check_update_with_no_data():
     """ Test when the partial fit is called without any data"""
     # Create an empty array
     prev_points = 100
@@ -156,8 +156,8 @@ def test_check_update_with_no_data():
     x_empty = np.empty((0, X.shape[1]))
     tmean, tvar = GaussianNB._update_mean_variance(prev_points, mean,
                                                    var, x_empty)
-    assert_equal(tmean, mean)
-    assert_equal(tvar, var)
+    assert tmean == mean
+    assert tvar == var
 
 
 def test_gnb_pfit_wrong_nb_features():
@@ -170,12 +170,257 @@ def test_gnb_pfit_wrong_nb_features():
     assert_raises(ValueError, clf.partial_fit, np.hstack((X, X)), y)
 
 
-def test_discrete_prior():
+def test_gnb_partial_fit():
+    clf = GaussianNB().fit(X, y)
+    clf_pf = GaussianNB().partial_fit(X, y, np.unique(y))
+    assert_array_almost_equal(clf.theta_, clf_pf.theta_)
+    assert_array_almost_equal(clf.sigma_, clf_pf.sigma_)
+    assert_array_almost_equal(clf.class_prior_, clf_pf.class_prior_)
+
+    clf_pf2 = GaussianNB().partial_fit(X[0::2, :], y[0::2], np.unique(y))
+    clf_pf2.partial_fit(X[1::2], y[1::2])
+    assert_array_almost_equal(clf.theta_, clf_pf2.theta_)
+    assert_array_almost_equal(clf.sigma_, clf_pf2.sigma_)
+    assert_array_almost_equal(clf.class_prior_, clf_pf2.class_prior_)
+
+
+def test_gnb_naive_bayes_scale_invariance():
+    # Scaling the data should not change the prediction results
+    iris = load_iris()
+    X, y = iris.data, iris.target
+    labels = [GaussianNB().fit(f * X, y).predict(f * X)
+              for f in [1E-10, 1, 1E10]]
+    assert_array_equal(labels[0], labels[1])
+    assert_array_equal(labels[1], labels[2])
+
+
+@pytest.mark.parametrize("cls", [MultinomialNB, BernoulliNB, CategoricalNB])
+def test_discretenb_prior(cls):
     # Test whether class priors are properly set.
-    for cls in [BernoulliNB, MultinomialNB]:
-        clf = cls().fit(X2, y2)
-        assert_array_almost_equal(np.log(np.array([2, 2, 2]) / 6.0),
-                                  clf.class_log_prior_, 8)
+    clf = cls().fit(X2, y2)
+    assert_array_almost_equal(np.log(np.array([2, 2, 2]) / 6.0),
+                              clf.class_log_prior_, 8)
+
+
+@pytest.mark.parametrize("cls", [MultinomialNB, BernoulliNB, CategoricalNB])
+def test_discretenb_partial_fit(cls):
+    clf1 = cls()
+    clf1.fit([[0, 1], [1, 0], [1, 1]], [0, 1, 1])
+
+    clf2 = cls()
+    clf2.partial_fit([[0, 1], [1, 0], [1, 1]], [0, 1, 1], classes=[0, 1])
+    assert_array_equal(clf1.class_count_, clf2.class_count_)
+    if cls is CategoricalNB:
+        for i in range(len(clf1.category_count_)):
+            assert_array_equal(clf1.category_count_[i],
+                               clf2.category_count_[i])
+    else:
+        assert_array_equal(clf1.feature_count_, clf2.feature_count_)
+
+    clf3 = cls()
+    # all categories have to appear in the first partial fit
+    clf3.partial_fit([[0, 1]], [0], classes=[0, 1])
+    clf3.partial_fit([[1, 0]], [1])
+    clf3.partial_fit([[1, 1]], [1])
+    assert_array_equal(clf1.class_count_, clf3.class_count_)
+    if cls is CategoricalNB:
+        # the categories for each feature of CategoricalNB are mapped to an
+        # index chronologically with each call of partial fit and therefore
+        # the category_count matrices cannot be compared for equality
+        for i in range(len(clf1.category_count_)):
+            assert_array_equal(clf1.category_count_[i].shape,
+                               clf3.category_count_[i].shape)
+            assert_array_equal(np.sum(clf1.category_count_[i], axis=1),
+                               np.sum(clf3.category_count_[i], axis=1))
+
+        # assert category 0 occurs 1x in the first class and 0x in the 2nd
+        # class
+        assert_array_equal(clf1.category_count_[0][0], np.array([1, 0]))
+        # assert category 1 occurs 0x in the first class and 2x in the 2nd
+        # class
+        assert_array_equal(clf1.category_count_[0][1], np.array([0, 2]))
+
+        # assert category 0 occurs 0x in the first class and 1x in the 2nd
+        # class
+        assert_array_equal(clf1.category_count_[1][0], np.array([0, 1]))
+        # assert category 1 occurs 1x in the first class and 1x in the 2nd
+        # class
+        assert_array_equal(clf1.category_count_[1][1], np.array([1, 1]))
+    else:
+        assert_array_equal(clf1.feature_count_, clf3.feature_count_)
+
+
+@pytest.mark.parametrize('cls', [BernoulliNB, MultinomialNB, GaussianNB,
+                                 CategoricalNB])
+def test_discretenb_pickle(cls):
+    # Test picklability of discrete naive Bayes classifiers
+
+    clf = cls().fit(X2, y2)
+    y_pred = clf.predict(X2)
+
+    store = BytesIO()
+    pickle.dump(clf, store)
+    clf = pickle.load(BytesIO(store.getvalue()))
+
+    assert_array_equal(y_pred, clf.predict(X2))
+
+    # Test pickling of estimator trained with partial_fit
+    clf2 = cls().partial_fit(X2[:3], y2[:3], classes=np.unique(y2))
+    clf2.partial_fit(X2[3:], y2[3:])
+    store = BytesIO()
+    pickle.dump(clf2, store)
+    clf2 = pickle.load(BytesIO(store.getvalue()))
+    assert_array_equal(y_pred, clf2.predict(X2))
+
+
+@pytest.mark.parametrize('cls', [BernoulliNB, MultinomialNB, GaussianNB,
+                                 CategoricalNB])
+def test_discretenb_input_check_fit(cls):
+    # Test input checks for the fit method
+
+    # check shape consistency for number of samples at fit time
+    assert_raises(ValueError, cls().fit, X2, y2[:-1])
+
+    # check shape consistency for number of input features at predict time
+    clf = cls().fit(X2, y2)
+    assert_raises(ValueError, clf.predict, X2[:, :-1])
+
+
+@pytest.mark.parametrize('cls', [BernoulliNB, MultinomialNB, CategoricalNB])
+def test_discretenb_input_check_partial_fit(cls):
+    # check shape consistency
+    assert_raises(ValueError, cls().partial_fit, X2, y2[:-1],
+                  classes=np.unique(y2))
+
+    # classes is required for first call to partial fit
+    assert_raises(ValueError, cls().partial_fit, X2, y2)
+
+    # check consistency of consecutive classes values
+    clf = cls()
+    clf.partial_fit(X2, y2, classes=np.unique(y2))
+    assert_raises(ValueError, clf.partial_fit, X2, y2,
+                  classes=np.arange(42))
+
+    # check consistency of input shape for partial_fit
+    assert_raises(ValueError, clf.partial_fit, X2[:, :-1], y2)
+
+    # check consistency of input shape for predict
+    assert_raises(ValueError, clf.predict, X2[:, :-1])
+
+
+def test_discretenb_predict_proba():
+    # Test discrete NB classes' probability scores
+
+    # The 100s below distinguish Bernoulli from multinomial.
+    # FIXME: write a test to show this.
+    X_bernoulli = [[1, 100, 0], [0, 1, 0], [0, 100, 1]]
+    X_multinomial = [[0, 1], [1, 3], [4, 0]]
+
+    # test binary case (1-d output)
+    y = [0, 0, 2]  # 2 is regression test for binary case, 02e673
+    for cls, X in zip([BernoulliNB, MultinomialNB],
+                      [X_bernoulli, X_multinomial]):
+        clf = cls().fit(X, y)
+        assert clf.predict(X[-1:]) == 2
+        assert clf.predict_proba([X[0]]).shape == (1, 2)
+        assert_array_almost_equal(clf.predict_proba(X[:2]).sum(axis=1),
+                                  np.array([1., 1.]), 6)
+
+    # test multiclass case (2-d output, must sum to one)
+    y = [0, 1, 2]
+    for cls, X in zip([BernoulliNB, MultinomialNB],
+                      [X_bernoulli, X_multinomial]):
+        clf = cls().fit(X, y)
+        assert clf.predict_proba(X[0:1]).shape == (1, 3)
+        assert clf.predict_proba(X[:2]).shape == (2, 3)
+        assert_almost_equal(np.sum(clf.predict_proba([X[1]])), 1)
+        assert_almost_equal(np.sum(clf.predict_proba([X[-1]])), 1)
+        assert_almost_equal(np.sum(np.exp(clf.class_log_prior_)), 1)
+        assert_almost_equal(np.sum(np.exp(clf.intercept_)), 1)
+
+
+@pytest.mark.parametrize('cls', [BernoulliNB, MultinomialNB, CategoricalNB])
+def test_discretenb_uniform_prior(cls):
+    # Test whether discrete NB classes fit a uniform prior
+    # when fit_prior=False and class_prior=None
+
+    clf = cls()
+    clf.set_params(fit_prior=False)
+    clf.fit([[0], [0], [1]], [0, 0, 1])
+    prior = np.exp(clf.class_log_prior_)
+    assert_array_almost_equal(prior, np.array([.5, .5]))
+
+
+@pytest.mark.parametrize('cls', [BernoulliNB, MultinomialNB, CategoricalNB])
+def test_discretenb_provide_prior(cls):
+    # Test whether discrete NB classes use provided prior
+
+    clf = cls(class_prior=[0.5, 0.5])
+    clf.fit([[0], [0], [1]], [0, 0, 1])
+    prior = np.exp(clf.class_log_prior_)
+    assert_array_almost_equal(prior, np.array([.5, .5]))
+
+    # Inconsistent number of classes with prior
+    assert_raises(ValueError, clf.fit, [[0], [1], [2]], [0, 1, 2])
+    assert_raises(ValueError, clf.partial_fit, [[0], [1]], [0, 1],
+                  classes=[0, 1, 1])
+
+
+@pytest.mark.parametrize('cls', [BernoulliNB, MultinomialNB, CategoricalNB])
+def test_discretenb_provide_prior_with_partial_fit(cls):
+    # Test whether discrete NB classes use provided prior
+    # when using partial_fit
+
+    iris = load_iris()
+    iris_data1, iris_data2, iris_target1, iris_target2 = train_test_split(
+        iris.data, iris.target, test_size=0.4, random_state=415)
+
+    for prior in [None, [0.3, 0.3, 0.4]]:
+        clf_full = cls(class_prior=prior)
+        clf_full.fit(iris.data, iris.target)
+        clf_partial = cls(class_prior=prior)
+        clf_partial.partial_fit(iris_data1, iris_target1,
+                                classes=[0, 1, 2])
+        clf_partial.partial_fit(iris_data2, iris_target2)
+        assert_array_almost_equal(clf_full.class_log_prior_,
+                                  clf_partial.class_log_prior_)
+
+
+@pytest.mark.parametrize('cls', [BernoulliNB, MultinomialNB, CategoricalNB])
+def test_discretenb_sample_weight_multiclass(cls):
+    # check shape consistency for number of samples at fit time
+    X = [
+        [0, 0, 1],
+        [0, 1, 1],
+        [0, 1, 1],
+        [1, 0, 0],
+    ]
+    y = [0, 0, 1, 2]
+    sample_weight = np.array([1, 1, 2, 2], dtype=np.float64)
+    sample_weight /= sample_weight.sum()
+    clf = cls().fit(X, y, sample_weight=sample_weight)
+    assert_array_equal(clf.predict(X), [0, 1, 1, 2])
+
+    # Check sample weight using the partial_fit method
+    clf = cls()
+    clf.partial_fit(X[:2], y[:2], classes=[0, 1, 2],
+                    sample_weight=sample_weight[:2])
+    clf.partial_fit(X[2:3], y[2:3], sample_weight=sample_weight[2:3])
+    clf.partial_fit(X[3:], y[3:], sample_weight=sample_weight[3:])
+    assert_array_equal(clf.predict(X), [0, 1, 1, 2])
+
+
+@pytest.mark.parametrize('cls', [BernoulliNB, MultinomialNB])
+def test_discretenb_coef_intercept_shape(cls):
+    # coef_ and intercept_ should have shapes as in other linear models.
+    # Non-regression test for issue #2127.
+    X = [[1, 0, 0], [1, 1, 1]]
+    y = [1, 2]  # binary classification
+    clf = cls()
+
+    clf.fit(X, y)
+    assert clf.coef_.shape == (1, 3)
+    assert clf.intercept_.shape == (1,)
 
 
 @pytest.mark.parametrize('kind', ('dense', 'sparse'))
@@ -230,207 +475,34 @@ def test_mnnb(kind):
     assert_array_almost_equal(y_pred_log_proba3, y_pred_log_proba)
 
 
-def check_partial_fit(cls):
-    clf1 = cls()
-    clf1.fit([[0, 1], [1, 0]], [0, 1])
+def test_mnb_prior_unobserved_targets():
+    # test smoothing of prior for yet unobserved targets
 
-    clf2 = cls()
-    clf2.partial_fit([[0, 1], [1, 0]], [0, 1], classes=[0, 1])
-    assert_array_equal(clf1.class_count_, clf2.class_count_)
-    assert_array_equal(clf1.feature_count_, clf2.feature_count_)
+    # Create toy training data
+    X = np.array([[0, 1], [1, 0]])
+    y = np.array([0, 1])
 
-    clf3 = cls()
-    clf3.partial_fit([[0, 1]], [0], classes=[0, 1])
-    clf3.partial_fit([[1, 0]], [1])
-    assert_array_equal(clf1.class_count_, clf3.class_count_)
-    assert_array_equal(clf1.feature_count_, clf3.feature_count_)
+    clf = MultinomialNB()
+
+    assert_no_warnings(
+        clf.partial_fit, X, y, classes=[0, 1, 2]
+    )
+
+    assert clf.predict([[0, 1]]) == 0
+    assert clf.predict([[1, 0]]) == 1
+    assert clf.predict([[1, 1]]) == 0
+
+    # add a training example with previously unobserved class
+    assert_no_warnings(
+        clf.partial_fit, [[1, 1]], [2]
+    )
+
+    assert clf.predict([[0, 1]]) == 0
+    assert clf.predict([[1, 0]]) == 1
+    assert clf.predict([[1, 1]]) == 2
 
 
-@pytest.mark.parametrize("cls", [MultinomialNB, BernoulliNB])
-def test_discretenb_partial_fit(cls):
-    check_partial_fit(cls)
-
-
-def test_gnb_partial_fit():
-    clf = GaussianNB().fit(X, y)
-    clf_pf = GaussianNB().partial_fit(X, y, np.unique(y))
-    assert_array_almost_equal(clf.theta_, clf_pf.theta_)
-    assert_array_almost_equal(clf.sigma_, clf_pf.sigma_)
-    assert_array_almost_equal(clf.class_prior_, clf_pf.class_prior_)
-
-    clf_pf2 = GaussianNB().partial_fit(X[0::2, :], y[0::2], np.unique(y))
-    clf_pf2.partial_fit(X[1::2], y[1::2])
-    assert_array_almost_equal(clf.theta_, clf_pf2.theta_)
-    assert_array_almost_equal(clf.sigma_, clf_pf2.sigma_)
-    assert_array_almost_equal(clf.class_prior_, clf_pf2.class_prior_)
-
-
-@pytest.mark.parametrize('cls', [BernoulliNB, MultinomialNB, GaussianNB])
-def test_discretenb_pickle(cls):
-    # Test picklability of discrete naive Bayes classifiers
-
-    clf = cls().fit(X2, y2)
-    y_pred = clf.predict(X2)
-
-    store = BytesIO()
-    pickle.dump(clf, store)
-    clf = pickle.load(BytesIO(store.getvalue()))
-
-    assert_array_equal(y_pred, clf.predict(X2))
-
-    if cls is not GaussianNB:
-        # TODO re-enable me when partial_fit is implemented for GaussianNB
-
-        # Test pickling of estimator trained with partial_fit
-        clf2 = cls().partial_fit(X2[:3], y2[:3], classes=np.unique(y2))
-        clf2.partial_fit(X2[3:], y2[3:])
-        store = BytesIO()
-        pickle.dump(clf2, store)
-        clf2 = pickle.load(BytesIO(store.getvalue()))
-        assert_array_equal(y_pred, clf2.predict(X2))
-
-
-@pytest.mark.parametrize('cls', [BernoulliNB, MultinomialNB, GaussianNB])
-def test_input_check_fit(cls):
-    # Test input checks for the fit method
-
-    # check shape consistency for number of samples at fit time
-    assert_raises(ValueError, cls().fit, X2, y2[:-1])
-
-    # check shape consistency for number of input features at predict time
-    clf = cls().fit(X2, y2)
-    assert_raises(ValueError, clf.predict, X2[:, :-1])
-
-
-@pytest.mark.parametrize('cls', [BernoulliNB, MultinomialNB])
-def test_input_check_partial_fit(cls):
-    # check shape consistency
-    assert_raises(ValueError, cls().partial_fit, X2, y2[:-1],
-                  classes=np.unique(y2))
-
-    # classes is required for first call to partial fit
-    assert_raises(ValueError, cls().partial_fit, X2, y2)
-
-    # check consistency of consecutive classes values
-    clf = cls()
-    clf.partial_fit(X2, y2, classes=np.unique(y2))
-    assert_raises(ValueError, clf.partial_fit, X2, y2,
-                  classes=np.arange(42))
-
-    # check consistency of input shape for partial_fit
-    assert_raises(ValueError, clf.partial_fit, X2[:, :-1], y2)
-
-    # check consistency of input shape for predict
-    assert_raises(ValueError, clf.predict, X2[:, :-1])
-
-
-def test_discretenb_predict_proba():
-    # Test discrete NB classes' probability scores
-
-    # The 100s below distinguish Bernoulli from multinomial.
-    # FIXME: write a test to show this.
-    X_bernoulli = [[1, 100, 0], [0, 1, 0], [0, 100, 1]]
-    X_multinomial = [[0, 1], [1, 3], [4, 0]]
-
-    # test binary case (1-d output)
-    y = [0, 0, 2]   # 2 is regression test for binary case, 02e673
-    for cls, X in zip([BernoulliNB, MultinomialNB],
-                      [X_bernoulli, X_multinomial]):
-        clf = cls().fit(X, y)
-        assert_equal(clf.predict(X[-1:]), 2)
-        assert_equal(clf.predict_proba([X[0]]).shape, (1, 2))
-        assert_array_almost_equal(clf.predict_proba(X[:2]).sum(axis=1),
-                                  np.array([1., 1.]), 6)
-
-    # test multiclass case (2-d output, must sum to one)
-    y = [0, 1, 2]
-    for cls, X in zip([BernoulliNB, MultinomialNB],
-                      [X_bernoulli, X_multinomial]):
-        clf = cls().fit(X, y)
-        assert_equal(clf.predict_proba(X[0:1]).shape, (1, 3))
-        assert_equal(clf.predict_proba(X[:2]).shape, (2, 3))
-        assert_almost_equal(np.sum(clf.predict_proba([X[1]])), 1)
-        assert_almost_equal(np.sum(clf.predict_proba([X[-1]])), 1)
-        assert_almost_equal(np.sum(np.exp(clf.class_log_prior_)), 1)
-        assert_almost_equal(np.sum(np.exp(clf.intercept_)), 1)
-
-
-@pytest.mark.parametrize('cls', [BernoulliNB, MultinomialNB])
-def test_discretenb_uniform_prior(cls):
-    # Test whether discrete NB classes fit a uniform prior
-    # when fit_prior=False and class_prior=None
-
-    clf = cls()
-    clf.set_params(fit_prior=False)
-    clf.fit([[0], [0], [1]], [0, 0, 1])
-    prior = np.exp(clf.class_log_prior_)
-    assert_array_almost_equal(prior, np.array([.5, .5]))
-
-
-@pytest.mark.parametrize('cls', [BernoulliNB, MultinomialNB])
-def test_discretenb_provide_prior(cls):
-    # Test whether discrete NB classes use provided prior
-
-    clf = cls(class_prior=[0.5, 0.5])
-    clf.fit([[0], [0], [1]], [0, 0, 1])
-    prior = np.exp(clf.class_log_prior_)
-    assert_array_almost_equal(prior, np.array([.5, .5]))
-
-    # Inconsistent number of classes with prior
-    assert_raises(ValueError, clf.fit, [[0], [1], [2]], [0, 1, 2])
-    assert_raises(ValueError, clf.partial_fit, [[0], [1]], [0, 1],
-                  classes=[0, 1, 1])
-
-
-@pytest.mark.parametrize('cls', [BernoulliNB, MultinomialNB])
-def test_discretenb_provide_prior_with_partial_fit(cls):
-    # Test whether discrete NB classes use provided prior
-    # when using partial_fit
-
-    iris = load_iris()
-    iris_data1, iris_data2, iris_target1, iris_target2 = train_test_split(
-        iris.data, iris.target, test_size=0.4, random_state=415)
-
-    for prior in [None, [0.3, 0.3, 0.4]]:
-        clf_full = cls(class_prior=prior)
-        clf_full.fit(iris.data, iris.target)
-        clf_partial = cls(class_prior=prior)
-        clf_partial.partial_fit(iris_data1, iris_target1,
-                                classes=[0, 1, 2])
-        clf_partial.partial_fit(iris_data2, iris_target2)
-        assert_array_almost_equal(clf_full.class_log_prior_,
-                                  clf_partial.class_log_prior_)
-
-
-@pytest.mark.parametrize('cls', [BernoulliNB, MultinomialNB])
-def test_sample_weight_multiclass(cls):
-    # check shape consistency for number of samples at fit time
-    check_sample_weight_multiclass(cls)
-
-
-def check_sample_weight_multiclass(cls):
-    X = [
-        [0, 0, 1],
-        [0, 1, 1],
-        [0, 1, 1],
-        [1, 0, 0],
-    ]
-    y = [0, 0, 1, 2]
-    sample_weight = np.array([1, 1, 2, 2], dtype=np.float64)
-    sample_weight /= sample_weight.sum()
-    clf = cls().fit(X, y, sample_weight=sample_weight)
-    assert_array_equal(clf.predict(X), [0, 1, 1, 2])
-
-    # Check sample weight using the partial_fit method
-    clf = cls()
-    clf.partial_fit(X[:2], y[:2], classes=[0, 1, 2],
-                    sample_weight=sample_weight[:2])
-    clf.partial_fit(X[2:3], y[2:3], sample_weight=sample_weight[2:3])
-    clf.partial_fit(X[3:], y[3:], sample_weight=sample_weight[3:])
-    assert_array_equal(clf.predict(X), [0, 1, 1, 2])
-
-
-def test_sample_weight_mnb():
+def test_mnb_sample_weight():
     clf = MultinomialNB()
     clf.fit([[1, 2], [1, 2], [1, 0]],
             [0, 0, 1],
@@ -441,75 +513,6 @@ def test_sample_weight_mnb():
                               [1 / 3., 2 / 3.])
 
 
-def test_coef_intercept_shape():
-    # coef_ and intercept_ should have shapes as in other linear models.
-    # Non-regression test for issue #2127.
-    X = [[1, 0, 0], [1, 1, 1]]
-    y = [1, 2]  # binary classification
-
-    for clf in [MultinomialNB(), BernoulliNB()]:
-        clf.fit(X, y)
-        assert_equal(clf.coef_.shape, (1, 3))
-        assert_equal(clf.intercept_.shape, (1,))
-
-
-def test_check_accuracy_on_digits():
-    # Non regression test to make sure that any further refactoring / optim
-    # of the NB models do not harm the performance on a slightly non-linearly
-    # separable dataset
-    digits = load_digits()
-    X, y = digits.data, digits.target
-    binary_3v8 = np.logical_or(digits.target == 3, digits.target == 8)
-    X_3v8, y_3v8 = X[binary_3v8], y[binary_3v8]
-
-    # Multinomial NB
-    scores = cross_val_score(MultinomialNB(alpha=10), X, y, cv=10)
-    assert_greater(scores.mean(), 0.86)
-
-    scores = cross_val_score(MultinomialNB(alpha=10), X_3v8, y_3v8, cv=10)
-    assert_greater(scores.mean(), 0.94)
-
-    # Bernoulli NB
-    scores = cross_val_score(BernoulliNB(alpha=10), X > 4, y, cv=10)
-    assert_greater(scores.mean(), 0.83)
-
-    scores = cross_val_score(BernoulliNB(alpha=10), X_3v8 > 4, y_3v8, cv=10)
-    assert_greater(scores.mean(), 0.92)
-
-    # Gaussian NB
-    scores = cross_val_score(GaussianNB(), X, y, cv=10)
-    assert_greater(scores.mean(), 0.77)
-
-    scores = cross_val_score(GaussianNB(var_smoothing=0.1), X, y, cv=10)
-    assert_greater(scores.mean(), 0.89)
-
-    scores = cross_val_score(GaussianNB(), X_3v8, y_3v8, cv=10)
-    assert_greater(scores.mean(), 0.86)
-
-
-def test_feature_log_prob_bnb():
-    # Test for issue #4268.
-    # Tests that the feature log prob value computed by BernoulliNB when
-    # alpha=1.0 is equal to the expression given in Manning, Raghavan,
-    # and Schuetze's "Introduction to Information Retrieval" book:
-    # https://nlp.stanford.edu/IR-book/html/htmledition/the-bernoulli-model-1.html
-
-    X = np.array([[0, 0, 0], [1, 1, 0], [0, 1, 0], [1, 0, 1], [0, 1, 0]])
-    Y = np.array([0, 0, 1, 2, 2])
-
-    # Fit Bernoulli NB w/ alpha = 1.0
-    clf = BernoulliNB(alpha=1.0)
-    clf.fit(X, Y)
-
-    # Manually form the (log) numerator and denominator that
-    # constitute P(feature presence | class)
-    num = np.log(clf.feature_count_ + 1.0)
-    denom = np.tile(np.log(clf.class_count_ + 2.0), (X.shape[1], 1)).T
-
-    # Check manual estimate matches
-    assert_array_almost_equal(clf.feature_log_prob_, (num - denom))
-
-
 def test_bnb():
     # Tests that BernoulliNB when alpha=1.0 gives the same values as
     # those given for the toy example in Manning, Raghavan, and
@@ -541,7 +544,8 @@ def test_bnb():
 
     # Check the feature probabilities are correct
     feature_prob = np.array([[0.4, 0.8, 0.2, 0.4, 0.4, 0.2],
-                             [1/3.0, 2/3.0, 2/3.0, 1/3.0, 1/3.0, 2/3.0]])
+                             [1 / 3.0, 2 / 3.0, 2 / 3.0, 1 / 3.0, 1 / 3.0,
+                              2 / 3.0]])
     assert_array_almost_equal(np.exp(clf.feature_log_prob_), feature_prob)
 
     # Testing data point is:
@@ -555,6 +559,29 @@ def test_bnb():
     assert_array_almost_equal(clf.predict_proba(X_test), predict_proba)
 
 
+def test_bnb_feature_log_prob():
+    # Test for issue #4268.
+    # Tests that the feature log prob value computed by BernoulliNB when
+    # alpha=1.0 is equal to the expression given in Manning, Raghavan,
+    # and Schuetze's "Introduction to Information Retrieval" book:
+    # http://nlp.stanford.edu/IR-book/html/htmledition/the-bernoulli-model-1.html
+
+    X = np.array([[0, 0, 0], [1, 1, 0], [0, 1, 0], [1, 0, 1], [0, 1, 0]])
+    Y = np.array([0, 0, 1, 2, 2])
+
+    # Fit Bernoulli NB w/ alpha = 1.0
+    clf = BernoulliNB(alpha=1.0)
+    clf.fit(X, Y)
+
+    # Manually form the (log) numerator and denominator that
+    # constitute P(feature presence | class)
+    num = np.log(clf.feature_count_ + 1.0)
+    denom = np.tile(np.log(clf.class_count_ + 2.0), (X.shape[1], 1)).T
+
+    # Check manual estimate matches
+    assert_array_almost_equal(clf.feature_log_prob_, (num - denom))
+
+
 def test_cnb():
     # Tests ComplementNB when alpha=1.0 for the toy example in Manning,
     # Raghavan, and Schuetze's "Introduction to Information Retrieval" book:
@@ -621,14 +648,56 @@ def test_cnb():
     assert_array_almost_equal(clf.feature_log_prob_, normed_weights)
 
 
-def test_naive_bayes_scale_invariance():
-    # Scaling the data should not change the prediction results
-    iris = load_iris()
-    X, y = iris.data, iris.target
-    labels = [GaussianNB().fit(f * X, y).predict(f * X)
-              for f in [1E-10, 1, 1E10]]
-    assert_array_equal(labels[0], labels[1])
-    assert_array_equal(labels[1], labels[2])
+def test_categoricalnb():
+    # Check the ability to predict the training set.
+    clf = CategoricalNB()
+    y_pred = clf.fit(X2, y2).predict(X2)
+    assert_array_equal(y_pred, y2)
+
+    X3 = np.array([[1, 4], [2, 5]])
+    y3 = np.array([1, 2])
+    clf = CategoricalNB(alpha=1, fit_prior=False)
+
+    clf.fit(X3, y3)
+
+    # Check error is raised for X with negative entries
+    X = np.array([[0, -1]])
+    y = np.array([1])
+    error_msg = "Negative values in data passed to CategoricalNB (input X)"
+    assert_raise_message(ValueError, error_msg, clf.predict, X)
+    assert_raise_message(ValueError, error_msg, clf.fit, X, y)
+
+    # Check error is raised for incorrect X
+    X = np.array([[1, 4, 1], [2, 5, 6]])
+    msg = "Expected input with 2 features, got 3 instead"
+    assert_raise_message(ValueError, msg, clf.predict, X)
+
+    # Test alpha
+    X3_test = np.array([[2, 5]])
+    # alpha=1 increases the count of all categories by one so the final
+    # probability for each category is not 50/50 but 1/3 to 2/3
+    bayes_numerator = np.array([[1/3*1/3, 2/3*2/3]])
+    bayes_denominator = bayes_numerator.sum()
+    assert_array_almost_equal(clf.predict_proba(X3_test),
+                              bayes_numerator / bayes_denominator)
+
+    # Assert category_count has counted all features
+    assert len(clf.category_count_) == X3.shape[1]
+
+    # Check sample_weight
+    X = np.array([[0, 0], [0, 1], [0, 0], [1, 1]])
+    y = np.array([1, 1, 2, 2])
+    clf = CategoricalNB(alpha=1, fit_prior=False)
+    clf.fit(X, y)
+    assert_array_equal(clf.predict(np.array([[0, 0]])), np.array([1]))
+
+    for factor in [1., 0.3, 5, 0.0001]:
+        X = np.array([[0, 0], [0, 1], [0, 0], [1, 1]])
+        y = np.array([1, 1, 2, 2])
+        sample_weight = np.array([1, 1, 10, 0.1]) * factor
+        clf = CategoricalNB(alpha=1, fit_prior=False)
+        clf.fit(X, y, sample_weight=sample_weight)
+        assert_array_equal(clf.predict(np.array([[0, 0]])), np.array([2]))
 
 
 def test_alpha():
@@ -644,7 +713,12 @@ def test_alpha():
     nb = MultinomialNB(alpha=0.)
     assert_warns(UserWarning, nb.partial_fit, X, y, classes=[0, 1])
     assert_warns(UserWarning, nb.fit, X, y)
-    prob = np.array([[2./3, 1./3], [0, 1]])
+    prob = np.array([[2. / 3, 1. / 3], [0, 1]])
+    assert_array_almost_equal(nb.predict_proba(X), prob)
+
+    nb = CategoricalNB(alpha=0.)
+    assert_warns(UserWarning, nb.fit, X, y)
+    prob = np.array([[1., 0.], [0., 1.]])
     assert_array_almost_equal(nb.predict_proba(X), prob)
 
     # Test sparse X
@@ -656,7 +730,7 @@ def test_alpha():
 
     nb = MultinomialNB(alpha=0.)
     assert_warns(UserWarning, nb.fit, X, y)
-    prob = np.array([[2./3, 1./3], [0, 1]])
+    prob = np.array([[2. / 3, 1. / 3], [0, 1]])
     assert_array_almost_equal(nb.predict_proba(X), prob)
 
     # Test for alpha < 0
@@ -666,8 +740,10 @@ def test_alpha():
                     'alpha should be > 0.')
     b_nb = BernoulliNB(alpha=-0.1)
     m_nb = MultinomialNB(alpha=-0.1)
+    c_nb = CategoricalNB(alpha=-0.1)
     assert_raise_message(ValueError, expected_msg, b_nb.fit, X, y)
     assert_raise_message(ValueError, expected_msg, m_nb.fit, X, y)
+    assert_raise_message(ValueError, expected_msg, c_nb.fit, X, y)
 
     b_nb = BernoulliNB(alpha=-0.1)
     m_nb = MultinomialNB(alpha=-0.1)
@@ -717,3 +793,52 @@ def test_alpha_vector():
     expected_msg = ('alpha should be a scalar or a numpy array '
                     'with shape [n_features]')
     assert_raise_message(ValueError, expected_msg, m_nb.fit, X, y)
+
+
+def test_check_accuracy_on_digits():
+    # Non regression test to make sure that any further refactoring / optim
+    # of the NB models do not harm the performance on a slightly non-linearly
+    # separable dataset
+    X, y = load_digits(return_X_y=True)
+    binary_3v8 = np.logical_or(y == 3, y == 8)
+    X_3v8, y_3v8 = X[binary_3v8], y[binary_3v8]
+
+    # Multinomial NB
+    scores = cross_val_score(MultinomialNB(alpha=10), X, y, cv=10)
+    assert scores.mean() > 0.86
+
+    scores = cross_val_score(MultinomialNB(alpha=10), X_3v8, y_3v8, cv=10)
+    assert scores.mean() > 0.94
+
+    # Bernoulli NB
+    scores = cross_val_score(BernoulliNB(alpha=10), X > 4, y, cv=10)
+    assert scores.mean() > 0.83
+
+    scores = cross_val_score(BernoulliNB(alpha=10), X_3v8 > 4, y_3v8, cv=10)
+    assert scores.mean() > 0.92
+
+    # Gaussian NB
+    scores = cross_val_score(GaussianNB(), X, y, cv=10)
+    assert scores.mean() > 0.77
+
+    scores = cross_val_score(GaussianNB(var_smoothing=0.1), X, y, cv=10)
+    assert scores.mean() > 0.89
+
+    scores = cross_val_score(GaussianNB(), X_3v8, y_3v8, cv=10)
+    assert scores.mean() > 0.86
+
+
+# TODO: remove in 0.24
+def test_deprecations():
+
+    class A(BaseNB, GaussianNB):
+        pass
+
+    class B(BaseDiscreteNB, CategoricalNB):
+        pass
+
+    with pytest.warns(FutureWarning, match="is deprecated in version 0.22"):
+        A()
+
+    with pytest.warns(FutureWarning, match="is deprecated in version 0.22"):
+        B()
