@@ -15,7 +15,7 @@ def test_compute_class_weight():
     y = np.asarray([2, 2, 2, 3, 3, 4])
     classes = np.unique(y)
 
-    cw = compute_class_weight("balanced", classes, y)
+    cw = compute_class_weight("balanced", classes=classes, y=y)
     # total effect of samples is preserved
     class_counts = np.bincount(y)[2:]
     assert_almost_equal(np.dot(cw, class_counts), y.shape[0])
@@ -27,25 +27,25 @@ def test_compute_class_weight_not_present():
     classes = np.arange(4)
     y = np.asarray([0, 0, 0, 1, 1, 2])
     with pytest.raises(ValueError):
-        compute_class_weight("balanced", classes, y)
+        compute_class_weight("balanced", classes=classes, y=y)
     # Fix exception in error message formatting when missing label is a string
     # https://github.com/scikit-learn/scikit-learn/issues/8312
     with pytest.raises(ValueError,
                        match="Class label label_not_present not present"):
-        compute_class_weight({"label_not_present": 1.}, classes, y)
+        compute_class_weight({"label_not_present": 1.}, classes=classes, y=y)
     # Raise error when y has items not in classes
     classes = np.arange(2)
     with pytest.raises(ValueError):
-        compute_class_weight("balanced", classes, y)
+        compute_class_weight("balanced", classes=classes, y=y)
     with pytest.raises(ValueError):
-        compute_class_weight({0: 1., 1: 2.}, classes, y)
+        compute_class_weight({0: 1., 1: 2.}, classes=classes, y=y)
 
 
 def test_compute_class_weight_dict():
     classes = np.arange(3)
     class_weights = {0: 1.0, 1: 2.0, 2: 3.0}
     y = np.asarray([0, 0, 1, 2])
-    cw = compute_class_weight(class_weights, classes, y)
+    cw = compute_class_weight(class_weights, classes=classes, y=y)
 
     # When the user specifies class weights, compute_class_weights should just
     # return them.
@@ -56,12 +56,12 @@ def test_compute_class_weight_dict():
     msg = 'Class label 4 not present.'
     class_weights = {0: 1.0, 1: 2.0, 2: 3.0, 4: 1.5}
     with pytest.raises(ValueError, match=msg):
-        compute_class_weight(class_weights, classes, y)
+        compute_class_weight(class_weights, classes=classes, y=y)
 
     msg = 'Class label -1 not present.'
     class_weights = {-1: 5.0, 0: 1.0, 1: 2.0, 2: 3.0}
     with pytest.raises(ValueError, match=msg):
-        compute_class_weight(class_weights, classes, y)
+        compute_class_weight(class_weights, classes=classes, y=y)
 
 
 def test_compute_class_weight_invariance():
@@ -98,14 +98,14 @@ def test_compute_class_weight_balanced_negative():
     classes = np.array([-2, -1, 0])
     y = np.asarray([-1, -1, 0, 0, -2, -2])
 
-    cw = compute_class_weight("balanced", classes, y)
+    cw = compute_class_weight("balanced", classes=classes, y=y)
     assert len(cw) == len(classes)
     assert_array_almost_equal(cw, np.array([1., 1., 1.]))
 
     # Test with unbalanced class labels.
     y = np.asarray([-1, 0, 0, -2, -2, -2])
 
-    cw = compute_class_weight("balanced", classes, y)
+    cw = compute_class_weight("balanced", classes=classes, y=y)
     assert len(cw) == len(classes)
     class_counts = np.bincount(y + 2)
     assert_almost_equal(np.dot(cw, class_counts), y.shape[0])
@@ -117,7 +117,7 @@ def test_compute_class_weight_balanced_unordered():
     classes = np.array([1, 0, 3])
     y = np.asarray([1, 0, 0, 3, 3, 3])
 
-    cw = compute_class_weight("balanced", classes, y)
+    cw = compute_class_weight("balanced", classes=classes, y=y)
     class_counts = np.bincount(y)[classes]
     assert_almost_equal(np.dot(cw, class_counts), y.shape[0])
     assert_array_almost_equal(cw, [2., 1., 2. / 3])
@@ -131,16 +131,16 @@ def test_compute_class_weight_default():
     classes_len = len(classes)
 
     # Test for non specified weights
-    cw = compute_class_weight(None, classes, y)
+    cw = compute_class_weight(None, classes=classes, y=y)
     assert len(cw) == classes_len
     assert_array_almost_equal(cw, np.ones(3))
 
     # Tests for partly specified weights
-    cw = compute_class_weight({2: 1.5}, classes, y)
+    cw = compute_class_weight({2: 1.5}, classes=classes, y=y)
     assert len(cw) == classes_len
     assert_array_almost_equal(cw, [1.5, 1., 1.])
 
-    cw = compute_class_weight({2: 1.5, 4: 0.5}, classes, y)
+    cw = compute_class_weight({2: 1.5, 4: 0.5}, classes=classes, y=y)
     assert len(cw) == classes_len
     assert_array_almost_equal(cw, [1.5, 1., 0.5])
 
